from finalsa.common.models import (SqsReponse)
from typing import Dict, List, Union, Optional
from uuid import uuid4
from datetime import datetime, timezone
from abc import ABC, abstractmethod


class SqsService(ABC):

    @staticmethod
    def default_correlation_id():
        return str(uuid4())

    @abstractmethod
    def receive_messages(
            self,
            queue_url: str,
            max_number_of_messages: int = 1,
            wait_time_seconds: int = 1
    ) -> List[SqsReponse]:
        pass

    @abstractmethod
    def send_message(
            self,
            queue_url: str,
            data: Dict,
            id: str = None,
            topic: str = "default",
            correlation_id: str = None,
            timestamp: datetime = datetime.now(timezone.utc)
    ) -> None:
        pass

    @abstractmethod
    def send_raw_message(
            self,
            queue_url: str,
            data: Union[Dict, str],
            attributes: Optional[Dict] = {}
    ) -> None:
        pass

    @abstractmethod
    def delete_message(self, queue_url: str, receipt_handle: str) -> None:
        pass

    @abstractmethod
    def get_queue_arn(self, queue_url: str) -> str:
        pass

    @abstractmethod
    def get_queue_attributes(self, queue_url: str, ) -> Dict:
        pass

    @abstractmethod
    def get_queue_url(self, queue_name: str) -> str:
        pass



from finalsa.sqs.client.clients import SqsServiceImpl
from finalsa.sqs.client.interfaces import SqsService
from finalsa.sqs.client.exceptions import SqsException
from finalsa.sqs.client.tests import SqsServiceTest

__version__ = "0.0.5"


__all__ = [
    "SqsService",
    "SqsServiceImpl",
    "SqsException",
    "SqsServiceTest",
]



from finalsa.sqs.client import (
    __version__, SqsServiceImpl, SqsService, SqsException, SqsServiceTest
)
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../../')))


def test_version():
    assert __version__ is not None


def test_sqs_service_impl():
    assert SqsServiceImpl is not None


def test_sqs_service():
    assert SqsService is not None


def test_sqs_exception():
    assert SqsException is not None


def test_sqs_service_test():
    assert SqsServiceTest is not None


def test_sqs_service_receive_messages():
    service = SqsServiceTest()
    messages = service.receive_messages("test")
    assert messages is not None
    assert len(messages) == 0


def test_sqs_service_send_message():
    service = SqsServiceTest()
    service.send_message("test", {"test": "test"})
    messages = service.receive_messages("test")
    assert messages is not None
    assert len(messages) == 1


def test_sqs_service_send_raw_message():
    service = SqsServiceTest()
    service.send_raw_message("test", {"test": "test"}, {"test": "test"})
    messages = service.receive_messages("test")
    assert messages is not None
    assert len(messages) == 1


def test_sqs_service_delete_message():
    service = SqsServiceTest()
    service.send_message("test", {"test": "test"})
    messages = service.receive_messages("test")
    assert messages is not None
    assert len(messages) == 1
    service.delete_message("test", messages[0].receipt_handle)
    messages = service.receive_messages("test")
    assert messages is not None
    assert len(messages) == 0


def test_sqs_service_get_queue_arn():
    service = SqsServiceTest()
    arn = service.get_queue_arn("test")
    assert arn is not None


def test_consume_message():
    service = SqsServiceTest()
    service.send_message("test", {"test": "test"})
    messages = service.receive_messages("test")
    assert messages is not None


def test_get_queue_attributes():
    service = SqsServiceTest()
    attributes = service.get_queue_attributes("test")
    assert attributes is not None


def test_get_queue_url():
    service = SqsServiceTest()
    url = service.get_queue_url("test")
    assert url is not None
    assert url == "https://sqs.us-east-1.amazonaws.com/123456789012/test"



from finalsa.sqs.client.exceptions import SqsException
from finalsa.sqs.client.interfaces import SqsService
from finalsa.common.models import (SqsMessage, SqsReponse, to_message_attributes)
from json import dumps
from typing import Dict, List, Callable, Union, Optional
from uuid import UUID, uuid4
from datetime import datetime, timezone
import boto3
import botocore
import botocore.exceptions


class SqsServiceImpl(SqsService):

    def __init__(
        self,
        get_correlation_id: Callable[[], Union[str, UUID]] = None
    ) -> None:
        self.sqs = boto3.client('sqs')
        if get_correlation_id is not None:
            self.get_correlation_id = get_correlation_id
        else:
            self.get_correlation_id = self.__class__.default_correlation_id

    def receive_messages(
            self,
            queue_url: str,
            max_number_of_messages: int = 1,
            wait_time_seconds: int = 1
    ) -> List[SqsReponse]:
        try:
            response = self.sqs.receive_message(
                QueueUrl=queue_url,
                AttributeNames=['SentTimestamp'],
                MaxNumberOfMessages=max_number_of_messages,
                MessageAttributeNames=['All'],
                WaitTimeSeconds=wait_time_seconds
            )
            if 'Messages' not in response:
                return []
            messages = response['Messages']
            message_responses = []
            for message in messages:
                message_responses.append(
                    SqsReponse.from_boto_response(message))
            return message_responses
        except botocore.exceptions.ClientError as err:
            raise SqsException(err, err.response['Error']['Message'])
        except Exception as ex:
            raise SqsException(ex, "No se pudo recibir el mensaje de la cola")

    def send_message(
            self,
            queue_url: str,
            data: Dict,
            id: str = None,
            topic: str = "default",
            correlation_id: str = None,
            timestamp: datetime = datetime.now(timezone.utc)
    ) -> None:
        if id is None:
            id = str(uuid4())
        if correlation_id is None:
            correlation_id = str(self.get_correlation_id())

        message = SqsMessage(
            id=UUID(id),
            topic=topic,
            payload=dumps(data),
            correlation_id=correlation_id,
            timestamp=timestamp
        )
        self.send_raw_message(queue_url, message.to_dict(), {
            'topic': message.topic,
            'correlation_id': message.correlation_id,
        })

    def send_raw_message(
            self,
            queue_url: str,
            data: Union[Dict, str],
            attributes: Optional[Dict] = {}
    ) -> None:
        real_data = data
        if isinstance(data, dict):
            real_data = dumps(data)
        try:
            self.sqs.send_message(
                QueueUrl=queue_url,
                MessageAttributes=to_message_attributes(attributes),
                MessageBody=real_data
            )
        except botocore.exceptions.ClientError as err:
            raise SqsException(err, err.response['Error']['Message'])
        except Exception as ex:
            raise SqsException(ex, "No se pudo enviar el mensaje a la cola")

    def delete_message(self, queue_url: str, receipt_handle: str):
        try:
            self.sqs.delete_message(
                QueueUrl=queue_url,
                ReceiptHandle=receipt_handle
            )
        except botocore.exceptions.ClientError as err:
            raise SqsException(err, err.response['Error']['Message'])
        except Exception as ex:
            raise SqsException(
                ex, f"The message could not be deleted from the queue {receipt_handle}")

    def get_queue_arn(self, queue_url: str):
        response = self.get_queue_attributes(queue_url)
        return response['Attributes']['QueueArn']

    def get_queue_attributes(self, queue_url: str, ):
        response = self.sqs.get_queue_attributes(
            QueueUrl=queue_url,
            AttributeNames=[
                'All',
            ]
        )
        return response

    def get_queue_url(self, queue_name: str):
        response = self.sqs.get_queue_url(QueueName=queue_name)
        return response['QueueUrl']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re

from setuptools import setup

PACKAGE = "finalsa-sqs-client"
URL = "https://github.com/finalsa/sqs-client"
PACKAGE_FOLDER = "finalsa/sqs/client"

def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    with open(os.path.join(package, "__init__.py")) as f:
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", f.read()).group(1)


def get_long_description():
    """
    Return the README.
    """
    with open("README.md", encoding="utf8") as f:
        return f.read()


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [
        dirpath
        for dirpath, dirnames, filenames in os.walk(package)
        if os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]


setup(
    name=PACKAGE,
    version=get_version(PACKAGE_FOLDER),
    url=URL,
    license="MIT",
    description="An utils package for using sqs.",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    keywords=[
        "dynamodb",
    ],
    author="Luis Jimenez",
    author_email="luis@finalsa.com",
    packages=get_packages(PACKAGE_FOLDER),
    package_data={PACKAGE: ["py.typed"]},
    include_package_data=True,
    zip_safe=True,
    python_requires=">=3.8",
    data_files=[("", ["LICENSE.md"])],
    install_requires=[
        "boto3>=1.20.3",
        "finalsa-common-models>=0.0.4"
    ],
    extras_require={
    
    },
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



from finalsa.sqs.client.interfaces import SqsService
from finalsa.common.models import (SqsReponse, SqsMessage)
from typing import Dict, List, Callable, Union, Optional
from uuid import uuid4, UUID
from json import dumps
from datetime import datetime, timezone


class SqsServiceTest(SqsService):

    def __init__(
        self,
        get_correlation_id: Callable[[], Union[str, UUID]] = None
    ) -> None:
        self.messages = {}
        if get_correlation_id is not None:
            self.get_correlation_id = get_correlation_id
        else:
            self.get_correlation_id = self.__class__.default_correlation_id

    def receive_messages(
            self,
            queue_url: str,
            max_number_of_messages: int = 1,
            _: int = 1
    ) -> List[SqsReponse]:
        if queue_url not in self.messages:
            return []
        messages = self.messages[queue_url]
        if len(messages) == 0:
            return []
        message_responses = []
        while len(messages) > 0 and len(message_responses) < max_number_of_messages:
            message_responses.append(messages.pop(0))
        return message_responses

    def send_message(
            self,
            queue_url: str,
            data: Dict,
            id: str = None,
            topic: str = "default",
            correlation_id: str = None,
            timestamp: datetime = datetime.now(timezone.utc)
    ) -> None:
        if id is None:
            id = str(uuid4())
        if correlation_id is None:
            correlation_id = str(self.get_correlation_id())

        message = SqsMessage(
            id=UUID(id),
            topic=topic,
            payload=dumps(data),
            correlation_id=correlation_id,
            timestamp=timestamp
        )
        self.send_raw_message(queue_url, message.to_dict(), {
            'topic': message.topic,
            'correlation_id': message.correlation_id,
        })

    def send_raw_message(
            self,
            queue_url: str,
            data: Union[Dict, str],
            attributes: Optional[Dict] = {}
    ) -> None:
        if queue_url not in self.messages:
            self.messages[queue_url] = []

        if 'correlation_id' not in attributes:
            attributes['correlation_id'] = str(self.get_correlation_id())

        self.messages[queue_url].append(SqsReponse.from_boto_response({
            'MessageId': str(uuid4()),
            'ReceiptHandle': str(uuid4()),
            'MD5OfBody': str(uuid4()),
            'Body': dumps(data),
            'Attributes': attributes,
            'MessageAttributes': {
                'correlation_id': {'Type': 'String', 'Value': attributes['correlation_id']}
            }
        }))

    def delete_message(self, queue_url: str, receipt_handle: str) -> None:
        if queue_url not in self.messages:
            return
        messages = self.messages[queue_url]
        for i, message in enumerate(messages):
            if message.receipt_handle == receipt_handle:
                messages.pop(i)
                return

    def get_queue_arn(self, _: str) -> str:
        return 'arn:aws:sqs:us-east-1:123456789012:MyQueue'

    def get_queue_attributes(self, _: str, ) -> Dict:
        return {
            "QueueArn": "arn:aws:sqs:us-east-1:123456789012:MyQueue",
            "ApproximateNumberOfMessages": "0",
            "ApproximateNumberOfMessagesNotVisible": "0",
            "ApproximateNumberOfMessagesNotVisible": "0",
            "ApproximateNumberOfMessagesDelayed": "0",
        }

    def get_queue_url(self, queue_name: str) -> str:
        return f"https://sqs.us-east-1.amazonaws.com/123456789012/{queue_name}"
